/*
 * A resource offering support for directory operations.
 *
 * Copyright (C) 2021 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <fsserver/directory_object_interface.h>
#include <fsserver/directory_provider.h>
#include <fsserver/resource.h>



/* Support for providing access to directories. */

class DirectoryResource : public Resource, public DirectoryObject
{
protected:
    DirectoryProvider *_provider;

    /* Notification endpoint for event subscription. */

    unsigned int _endpoint;

public:
    fileid_t fileid;

    DirectoryResource(fileid_t fileid, DirectoryProvider *provider);

    virtual ~DirectoryResource();

    virtual void close();

    /* Server details. */

    int expected_items();

    ipc_server_handler_type handler();

    void *interface()
    { return static_cast<DirectoryObject *>(this); }

    /* Directory reading methods. */

    virtual long opendir(offset_t *size, l4_cap_idx_t *cap,
                         object_flags_t *object_flags);

    /* Notification methods. */

    virtual long subscribe(l4_cap_idx_t endpoint, notify_flags_t flags);
};

// vim: tabstop=4 expandtab shiftwidth=4
